/* Copyright (c) 2021 MIT 6.031 course staff, all rights reserved.
 * Redistribution of original or derived work requires permission of course staff.
 */

// this code is loaded into draw.html

const drawButton: HTMLButtonElement = document.getElementById('drawButton') as HTMLButtonElement;
const clearButton: HTMLButtonElement = document.getElementById('clearButton') as HTMLButtonElement;
const canvasElement: HTMLCanvasElement = document.getElementById('canvas') as HTMLCanvasElement;
const sizeBox: HTMLInputElement = document.getElementById('sizeBox') as HTMLInputElement;
const sizeSlider: HTMLInputElement = document.getElementById('sizeSlider') as HTMLInputElement;

drawButton.addEventListener('click', (event:MouseEvent) => {
    drawShape(canvasElement.width * Math.random(), canvasElement.height * Math.random());
    // for (let i = 0; i < 1000*1000; ++i) {
    //     drawShape(canvasElement.width * Math.random(), canvasElement.height * Math.random());
    // }
});

clearButton.addEventListener('click', clear);

// synchronize the size textbox and slider
sizeBox.addEventListener('input', () => {
    sizeSlider.value = sizeBox.value;
});
sizeSlider.addEventListener('input', () => {
    sizeBox.value = sizeSlider.value;
});

/**
 * @returns color shown in the color textbox
 */
function getColor():string {
    return (document.getElementById('colorTextbox') as HTMLInputElement).value; 
}

/**
 * @returns size shown in the size textbox
 */
 function getSize():number {
    return parseInt((document.getElementById('sizeBox') as HTMLInputElement).value); 
}

/**
 * Draw a square filled with a random color.
 * @param x x position of center of box centers
 * @param y y position of center of box centers
 */
function drawShape(x: number, y: number):void {
    const canvas = canvasElement.getContext('2d');
    if ( ! canvas ) throw new Error('unable to get canvas drawing context');

    // save original context settings before we translate and change colors
    canvas.save();

    // move origin to (x,y)
    canvas.translate(x, y);

    // draw the outer outline box centered on the origin (which is now x,y)
    canvas.strokeStyle = 'black';
    const size = getSize();
    canvas.lineWidth = 2;
    canvas.strokeRect( -size/2, -size/2, size, size);

    // fill with a random color
    canvas.fillStyle = getColor();
    canvas.fillRect( -size/2, -size/2, size, size);

    // reset the origin and styles back to defaults
    canvas.restore();
}

/**
 * Clear the drawing area, filling it with white again.
 */
function clear() {
    const context = canvasElement.getContext('2d');
    if ( ! context ) throw new Error('unable to get canvas drawing context');

    context.save();
    context.fillStyle = 'white';
    context.fillRect(0, 0, canvasElement.width, canvasElement.height);
    context.restore();
}